#!/usr/bin/python3
#
# Test program to generate some workload on a server.
#
import threading
import time
import psycopg2

connstr_template = "host=localhost dbname=postgres {} connect_timeout=3"
hosts = ["port=5400"]

def connect_to_primary():
    conn = ""

    while True:
        last_error = None
        for host in hosts:
            try:
                connstr = connstr_template.format(host)
                conn = psycopg2.connect(connstr)

                cur = conn.cursor()
                cur.execute("show transaction_read_only")
                if (cur.fetchone()[0] == 'on'):
                    pass
                else:
                    print("connected to {}".format(host))
                    return conn
            except Exception as e:
                print("connection failed to {}".format(host))
                last_error = e
        time.sleep(1);

    raise last_error

def create_table():
    conn = connect_to_primary()

    # Open a cursor to perform database operations
    cur = conn.cursor()

    # Execute a query
    cur.execute("DROP TABLE IF EXISTS testapp_tab");
    cur.execute("CREATE TABLE testapp_tab (id int PRIMARY KEY, updatecount int, balance int)")
    cur.execute("INSERT INTO testapp_tab SELECT g, 0, 0 FROM generate_series(1, 1000) g")
    conn.commit()

def print_table():
    conn = connect_to_primary()

    # Open a cursor to perform database operations
    cur = conn.cursor()

    # Execute a query
    cur.execute("SELECT SUM(updatecount) FROM testapp_tab");

    print ("SUM: {}".format(cur.fetchone()[0]))


def app_thread( threadId ):
    conn = connect_to_primary()

    # Open a cursor to perform database operations
    cur = conn.cursor()
    count = 0
    commitcount = 0
    while count < 100000:
        try:
            cur.execute("UPDATE testapp_tab SET updatecount = updatecount + 1, balance=balance-1 WHERE id = %s", (threadId,))
            cur.execute("UPDATE testapp_tab SET balance=balance + 1 WHERE id = %s*2 + 1", (threadId,))
            count = count + 1
            conn.commit()
            commitcount = commitcount + 1

            if (count % 100 == 0 and threadId == 1):
                cur.execute("CHECKPOINT")

            if (count % 100 == 10 and threadId == 2):
                cur.execute("CREATE TABLE foobar (i int)")
                print ("createed foobar")

            if (count % 100 == 20 and threadId == 2):
                cur.execute("INSERT INTO foobar select generate_series(1, 1000000)")

            if (count % 100 == 20 and threadId == 2):
                cur.execute("DELETE FROM foobar WHERE i > 100");

            if (count % 100 == 25 and threadId == 2):
                conn.commit()
                conn.set_session(autocommit=True)
                cur.execute("VACUUM foobar");
                conn.set_session(autocommit=False)
                
            if (count % 100 == 90 and threadId == 2):
                cur.execute("DROP TABLE foobar")
                print ("dropped foobar")
                
            if (count % 1000 == 0):
                print ("Thread has {} performed {}-{} updates".format(threadId, commitcount, count))
        except Exception as e:
            print("update failed: {}".format(e));
            conn = connect_to_primary()
            cur = conn.cursor()
    print ("Thread {} performed {}-{} updates".format(threadId, commitcount, count))

create_table()
      
# Create two threads as follows
threads = []
for threadno in range(0, 10):
    t = threading.Thread(target=app_thread, args=(threadno + 1,))
    t.start()
    threads.append(t)

for t in threads:
    t.join()

print_table()

